@@ -1,6 +1,6 @@
 /* Program invocation, startup and shutdown
 
-   Copyright (c) 1997-2011 Free Software Foundation, Inc.
+   Copyright (c) 1997-2012 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -37,7 +37,7 @@
 #include "extern.h"
 
 #define ZILE_COPYRIGHT_STRING \
-  "Copyright (C) 2011 Free Software Foundation, Inc."
+  "Copyright (C) 2012 Free Software Foundation, Inc."
 
 /* The current window; the first window in list. */
 Window *cur_wp = NULL, *head_wp = NULL;
@@ -338,9 +338,7 @@ main (int argc, char **argv)
   while (!(thisflag & FLAG_QUIT))
     {
       if (lastflag & FLAG_NEED_RESYNC)
-        resync_redisplay (cur_wp);
-      term_redisplay ();
-      term_refresh ();
+        window_resync (cur_wp);
       get_and_run_command ();
     }
 
