@@ -65,8 +65,12 @@ minibuf_refresh (void)
 static void
 minibuf_vwrite (const char *fmt, va_list ap)
 {
-  minibuf_contents = xvasprintf (fmt, ap);
-  minibuf_refresh ();
+  char *s = xvasprintf (fmt, ap);
+  if (minibuf_contents == NULL || strcmp (s, minibuf_contents))
+    {
+      minibuf_refresh ();
+      minibuf_contents = s;
+    }
 }
 
 /*
@@ -215,7 +219,7 @@ minibuf_read_yn (const char *fmt, ...)
     size_t key;
 
     minibuf_write ("%s%s", errmsg, buf);
-    key = getkey (GETKEY_DEFAULT);
+    key = getkeystroke (GETKEY_DEFAULT);
     switch (key)
       {
       case 'y':
